#include <stdrsl/Colors.h>

//A simple layer shader
class layeredShader(
    //Shader handles
    uniform string shader_one = "";
    uniform string shader_two = "";
    //Blending variables
    uniform float blend_intensity = 0.5;
    uniform string blend_map = "";
    //Other
    uniform float noise_blend = 1;
    uniform float bump_value = 0.015;
    )
{
  //Class members
  uniform shader m_upper_layer = null;
  uniform shader m_lower_layer = null;
  varying float m_blend;
  varying float m_rr;

  //Deterioration function to be used with mask
  float noise_mask()
  {
    varying point Pw = transform("object", P);

    varying float noise_one = (float noise(Pw) - 0.5) * 2;
    varying float noise_two = (float noise(Pw * 2) - 0.5);
    varying float noise_three = (float noise(Pw * 4) - 0.5) / 2;
    varying float fbm = (noise_one + noise_two + noise_three) * 0.5 + 0.5;
    varying float smoothstp = smoothstep(0.18, 0.27, fbm);

    return (1 - (1 - smoothstp) * (1 - step(0.45, float noise(Pw * 0.3))));
  }
  
  public void construct()
  {
    //No construct needed
  }

  //Shader methods
  public void begin()
  {
    //Load shaders
    if(shader_one != "")
    {
      m_upper_layer = getshader(shader_one);
      if(m_upper_layer != null)
        m_upper_layer->construct();
    }

    if(shader_two != "")
    {
      m_lower_layer = getshader(shader_two);
      if(m_lower_layer != null)
        m_lower_layer->construct();
    }

    //Get blend value
    m_blend = blend_intensity;
    if(blend_map != "")
      m_blend = float texture(blend_map);

    if(noise_blend != 0)
      m_blend = m_blend * noise_mask();

    //Using layer based on russian roulette 
    m_rr = (float random() > m_blend) ? 0 : 1;
    
    //Begin shaders
    if(m_rr != 0)
    {
      if(m_upper_layer != null)
        m_upper_layer->begin();
    }
    else
    {
      if(m_lower_layer != null)
        m_lower_layer->begin();
    }
  }

  public void displacement(output point P; output normal N)
  {
    //Calculate difference between shading and geometric normals (Beier:1999)
    normal diff = normalize(N) - normalize(Ng);

    //Get displaced normal from blend variable
    varying point Pt = P + N * (1 - m_blend) * bump_value * -1;
    varying normal Nf = normalize(calculatenormal(Pt));

    //Add difference back to normal
    N = Nf + diff;

    //Pass new noraml to each layer
    varying point Pul = P;
    varying normal Nul = N;
    if(m_upper_layer != null)
      m_upper_layer->displacement(Pul, Nul);

    varying point Pll = P;
    varying normal Nll = N;
    if(m_lower_layer != null)
      m_lower_layer->displacement(Pll, Nll);
  }

  public void prelighting(output color Ci, Oi;)
  {
    //No prelighting needed
  }

  public void lighting(output color Ci, Oi;)
  {
    //Run lighting methods on selected layer
    if(m_rr != 0)
    {
      if(m_upper_layer != null)
      {
        m_upper_layer->prelighting(Ci, Oi);
        m_upper_layer->lighting(Ci, Oi);
      }
    }
    else
    {
      if(m_lower_layer != null)
      {
        m_lower_layer->prelighting(Ci, Oi);
        m_lower_layer->lighting(Ci, Oi);
      }
    }
  }
}